import os
import argparse
import xarray as xr

from DHDL import subprogs
from DHDL import subprogs_xarray
from DHDL import get1varUV
from DHDL import vertical_cross_section as vcs

from DDRW import draw_hori_ver_2FIG as dvcs

####################
# developer shimizu shingo, NIED
##############################
# draw single vertical cross section and single horizontal distribution

def main():

    ## control for command line argument ##########################
    parser = argparse.ArgumentParser(description='draw netcdf')
    parser.add_argument('arg1',help='input file setting')
    parser.add_argument('arg2',help='variable setting')
    parser.add_argument('-t','--timenum',help='time slide from initial time')
    parser.add_argument('-T','--TIME',help='YYYY-M2-D2_H2:M2:S2(UTC)')
    parser.add_argument('-z','--levelnum',help='level number to be drawn')
    parser.add_argument('-Z','--LEV',help='level (m) to be drawn')
    parser.add_argument('-O','--OUTFIG',help='output file name')
    parser.add_argument('-W','--WIND',help='opt for wind draw 0:vector 1:barb')
    parser.add_argument('-WT','--WINDTHIN',\
                        help='data thinning out for vector, set mabiki factor')
    parser.add_argument('-CT','--ColorType',\
                        help='To use preset colormap, select colortype')
    parser.add_argument('-AR','--AutoRange',action='store_true',\
                        help='1: Manually set color range')
    parser.add_argument('-X0V','--X0_VCS',help='start lon for vcross section')
    parser.add_argument('-Y0V','--Y0_VCS',help='start lat for vcross section')
    parser.add_argument('-L0V','--L0_VCS',help='length(km) for vcross section')
    parser.add_argument('-A0V','--A0_VCS',\
                        help='deg anti-clockwise from east for vcross section')

    parser.add_argument('-TP','--TOPO',help='topography data is used to mask')
    parser.add_argument('-VT','--VERT_INTP',action='store_true',\
                        help='vertically interpolation will be applied')
    parser.add_argument('-JPN','--JapanTime',action='store_true',\
                        help='Time is converted into ')
    
    args = parser.parse_args()

    slon_ver = float(args.X0_VCS)
    slat_ver = float(args.Y0_VCS)
    len_ver  = float(args.L0_VCS)
    theta    = float(args.A0_VCS)

    if args.VERT_INTP:
        need_vertical_interp = "YES"
    else:
        need_vertical_interp = "NO"

    ####################################################################
    infile = args.arg1
    if not os.path.exists(infile):
        print ("infile does not exist \n",infile)
        exit(1)
        
    #open NETCDF as XARRAY
    data     = xr.open_dataset(infile)
    
    # get variable name
    varname  = args.arg2 
    
    #############################################################
    # check if the variable is included in NETCDF 
    subprogs.check_variable(data,varname,args)
    #############################################################

    #############################################################
    # check the dimension name    
    dimnameT,dimnameZ,dimnameY,dimnameX =subprogs.set_dim_name(data)
    #############################################################

    #############################################################
    # get lon and lat for vertical cross section
    dlon = subprogs.lon_per1km(data[dimnameY].values)
    elon_ver,elat_ver = vcs.change_input_ver_cross(slon_ver,slat_ver,\
                                                   len_ver,theta,dlon)

    vert_info = [slon_ver,slat_ver,elon_ver,elat_ver,dlon,theta]
    #############################################################
    # check vertical cross section within domain
    vcs.check_domain(vert_info,data)
    #############################################################

    if args.TOPO is not None:
        TOPO = subprogs.get_TOPOdata(data,"TOPO",args)
    else:
        TOPO = "NONE"
        
    ### MAKE VERTICAL CROSS SECTION ##########
    if args.WIND is not None:
        Z1D,VER,TTT,colormapV,clevelsV,UVALONG,WVER,scaleUV_V,maxUV_V =\
        vcs.make_vcross_sec(vert_info,data,varname,args,\
                            need_vertical_interp,TOPO)
    else:
        Z1D,VER,TTT,colormapV,clevelsV=\
        vcs.make_vcross_sec(vert_info,data,varname,args,\
                            need_vertical_interp,TOPO)
        
    #############################################################
    # get 2Dim for one variable with U and V wind vecotr    #####
    #############################################################
    if args.WIND is not None:
        V2D,colormapH,clevelsH,windU2D,windV2D,scaleUV_H,maxUV_H,TTT,ZZZ = \
            get1varUV.get1varUV(data,varname,args)
    else:
        V2D,colormapH,clevelsH,TTT,ZZZ = get1varUV.get1varUV(data,varname,args)

    #############################################################
    # draw region setting and check the domain 
    #############################################################
    slon = 137
    elon = 140
    slat = 35.0
    elat = 36.8
    LLL = subprogs.check_draw_domain(slon,elon,slat,elat,data)
    if LLL[0] == -1:
        slon = LLL[1]
        elon = LLL[2]
        slat = LLL[3]
        elat = LLL[4]
        
    #################################################################
    # description for figure is saved as "figure title" 
    FigTitle_V= "var= " + varname + "(" \
        + str(data[varname].units) + ")"
    FigTitle_H= subprogs.set_figtitle(args,data,infile,varname,TTT,ZZZ)
    ################################################################


    ### draw vertical cross section ####
    draw_top    = 0.0 # draw_top == draw_bottom : no limilation
    draw_bottom = 0.0 # when draw_top>draw_bottom: draw the vertical range

    # if draw_top< draw_bottom, set_extent will skip (whole domain draw)
    if args.WIND is not None:
        dvcs.draw_hori_ver_2FIG(slon,elon,slat,elat,data,V2D,\
                                colormapH,clevelsH,FigTitle_H,\
                                args,VER,FigTitle_V,\
                                colormapV,clevelsV,\
                                draw_top,draw_bottom,vert_info,\
                                windU2D,windV2D,scaleUV_H,\
                                maxUV_H,UVALONG,WVER,\
                                scaleUV_V,maxUV_V)
    else:
        dvcs.draw_hori_ver_2FIG(slon,elon,slat,elat,data,V2D,\
                                colormapH,clevelsH,FigTitle_H,\
                                args,VER,FigTitle_V,colormapV,clevelsV,\
                                draw_top,draw_bottom,vert_info)
    
if __name__ == "__main__":
    main()
